/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Url Copy: UrlCopy Error definitions
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: UrlCopyError.h,v 1.3 2007/03/08 12:44:46 ptedesco Exp $
 *  Release: $Name: glite-data-transfer-url-copy_R_2_0_1_6 $
 */
#ifndef GLITE_DATA_TRANSFER_URL_COPY_URL_COPY_ERROR_H_
#define GLITE_DATA_TRANSFER_URL_COPY_URL_COPY_ERROR_H_

#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace urlcopy {
	    
/**
 * UrlCopyError Definition
 */	
struct UrlCopyError {

    /**
     * The Error Scope values
     */    
    typedef enum{
        SCOPE_UNDEF   = 0,
        SCOPE_SOURCE     , // The error happened contacting the source SRM
        SCOPE_DESTINATION, // The error happened contacting the destination SRM
        SCOPE_TRANSFER   , // The error happened during the transfer
        SCOPE_AGENT        // The error is due to some internal agent problem (inconsistency, file system errors, etc)
    } Scope;
    
    /**
     * The Error Phase values
     */    
    typedef enum{
        PHASE_UNDEF    = 0,
        PHASE_PREPARATION , // The error happened during the praparation phase
        PHASE_FINALIZATION, // The error happened during the finalization phase
        PHASE_TRANSFER      // The error happened during the transfer phase
    } Phase;

    /**
     * The Error Category values
     */    
    typedef enum{
        SUCCESS            = 0,
        ERROR_USER            , // The user provided invalid data, e.g. invalid surl format
        ERROR_PERMISSION      , // Permission problem (Authentication or Authorization)
        ERROR_FILE_EXISTS     , // Destination file already exists
        ERROR_INVALID_PATH    , // Source file does not exist or is not possible to create destination directory path
        ERROR_NO_SPACE_LEFT   , // No space left on target device
        ERROR_GRIDFTP         , // Griftp failure
        ERROR_NETWORK         , // Network problem
        ERROR_INVALID_SIZE    , // Inconsistent file size at source and destination after transfer
        ERROR_REQUEST_FAILURE , // Failed to invoke remote method (RPC failure), e.g. SOAP_FAULT returned
        ERROR_CONNECTION      , // Connection error in contacting remote server (SRM or GRIDFTP)
        ERROR_HTTP_TIMEOUT    , // HTTP timeout 
        ERROR_REQUEST_TIMEOUT , // Timeout in completing asynchronous request
        ERROR_TRANSFER_TIMEOUT, // The transfer didn't complete in the expected time
        ERROR_MARKERS_TIMEOUT , // Transfer Markers Timeout
        ERROR_INTERNAL_ERROR  , // Agent internal error, e.g. Logic error in the agent
        ERROR_CONSISTENCY     , // Consistency problem with the transfer request, e.g. unsupported/old version of the Stat file
        ERROR_ABORTED         , // The request has been aborted
        ERROR_GENERAL_FAILURE , // Generic error, catch-all category
        ERROR_LOST              // The transfer was lost (dead, killed)
    } Category;

    /**
     * The Error Scope
     */
    Scope    scope;
    /**
     * The Error Phase
     */
    Phase    phase;
    /**
     * The Error category
     */
    Category category;
    /**
     * The Error reason description
     */
    std::string description;
    
    /**
     * Default Constructor
     */
    UrlCopyError():
        scope(SCOPE_UNDEF),
        phase(PHASE_UNDEF),
        category(SUCCESS){}
    /**
     * Constructor
     */
    UrlCopyError(Scope s, Phase p, Category c, const std::string& d):
        scope(s),
        phase(p),
        category(c),
        description(d){}
            
    /**
     * Return string representation of the Error Scope 
     */
    static const char * scopeToString(Scope value) throw();
    /**
     * Return string representation of the Error Phase
     */
    static const char * phaseToString(Phase value) throw();
    /**
     * Return string representation of the Error Category
     */
    static const char * categoryToString(Category value) throw();
};


} // end namespace urlcopy
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_URL_COPY_URL_COPY_ERROR_H_
